package simulation;

import java.util.Random;

public abstract class Animal  /*extends Thread*/ {	//abstract class of which specific animal are inherited.

	protected Random ranGen;
	protected int size;
	protected boolean carnivore;
	protected int energy; //cannot be seen outside this package.
	protected int animalUid;		
	
	protected Coordinates coord;
	
	public Animal( )	//constructor
	{
		ranGen = new Random();
		coord = new Coordinates();
	}

	public abstract String getSpecies();

	public void meet(Animal a) {
		if (this.carnivore) {
			if (a.carnivore) {
				if (a.getSpecies().equals(this.getSpecies())) {
					fightAgainst(a);
				} else { //fressi
					fightAgainst(a);
					if (getValue() > a.getValue()) {
						eat(a);
					} else if (getValue() < a.getValue()) {
						a.eat(this);
					} else {
						die();
						a.die();
					}
				}
			} else { // herbivore
				fightAgainst(a);
				eat(a);
			}
		} else {
			if (a.carnivore) {
				a.meet(this);
			} else {
				fightAgainst(a);			   
			}
		}
	}

	protected void changeEnergy(double changeEnergy)  {
		energy = (int)((double)energy + (double)changeEnergy);
	}

	public final boolean isAnimalAlive() {
		return energy > 0;
	}

	public final void die() {
		energy = 0;
	}

	public int getValue() {
		return energy * size;
	}

	public final void fightAgainst(Animal a) {
		double value = getValue();
		double enemyValue = a.getValue();
		double totalSystemValue = value + enemyValue; 

		changeEnergy(-this.energy * enemyValue / totalSystemValue);
		a.changeEnergy(-a.energy * value / totalSystemValue);
	}

	public void eat(Animal a) {
		changeEnergy(a.size / 2);
		a.die();
	}

	public String toString() {
		StringBuffer TheBuffer = new StringBuffer();   //StringBuffer for output
		TheBuffer.append(getSpecies() + " " + animalUid);

		if(this.isAnimalAlive()) {								//is the animal still alive??
			TheBuffer.append(" (Energy " + energy + ", Size "+ size + ", " + coord + ")");
		}
		else {
			TheBuffer.append(" is dead!");
		}

		return TheBuffer.toString();
	}

	public void move() {
		coord.move();
		int r = ranGen.nextInt(100);
		if (r < 5) {
			IO.writeLn(this.getSpecies() + " " + this.animalUid + " is starving alone in the savannah... energy -1.");
			changeEnergy(-1);
			if (!this.isAnimalAlive()) {
				//if it's dead, message
				IO.writeLn(this.toString());
			}
		}
	}

	public boolean isNear(Animal a) {
		return a.coord.equals(this.coord);
	}

	public void setEnergy(int e){        	//change energy
		this.energy = e;
	}

}