package simulation;

/**
 * The class SavannahSim handles the execution parameters, does the savannah
 * simulation and also offers methods to count the number of antilopes and lions
 * alive.
 * 
 * @author Carl Celimoll
 * @copyright Systems Simulation Inc. 2007
 * @history 2007-11-26 CC built the application. 
 * 			2007-11-29 CC documentations and review.
 * @version 2007-11-29 CC v0.9
 * 
 * @responsibilities CC
 * @see simulation.Animal
 * @see simulation.Antilope
 * @see simulation.Lion
 */

public class SavannahSim {

	private Animal[] animal;

	private int iterations = 0;

	public static void main(String[] args) {
		// also hand-over of two parameters when executing e.g. "java
		// SavannahSim 2 4", meaning: 2 Lions and 4 Antilopes.
		new SavannahSim(Integer.parseInt(args[0]), Integer.parseInt(args[1]));
	}
	
	/**
	 * Creates all the animals, lets the animals randomly meet each other until
	 * there is a winner, outputs all animals status.
	 * 
	 * @pre lions != null && antilopes != null;
	 * @post
	 * @param lions
	 *            The number of lions.
	 * @param antilopes
	 *            The numer of antilopes.
	 */
	public SavannahSim(int lions, int antilopes) {
		
		IO.writeLn("----------------------------------------------");
		IO.writeLn("Savannah Simulation, Version 0.9");
		IO.writeLn("Animals: Lions: " + lions + ", Antilopes: " + antilopes
				+ ".");
		IO.writeLn("----------------------------------------------");
		animal = new Animal[lions + antilopes];

		// creating lions and antilopes
		for (int i = 0; i < lions; i++) {
			animal[i] = new Lion(i + 1/*, s*/);
		}
		for (int i = lions; i < (lions + antilopes); i++) {
			animal[i] = new Antilope(i + 1 - lions/*, s*/);
		}

		// simulation as long there isn't a winner.
		while (livingAnimals() > 1) {
			for (int i = 0; i < animal.length; i++) {
				if (animal[i].isAnimalAlive()) {
					IO.writeLn("Moving: " + animal[i]);
					animal[i].move();
				}
				for (int k = 0; k < animal.length; k++ ) {
					if (k != i && animal[i].isAnimalAlive() && animal[k].isAnimalAlive() && animal[i].isNear(animal[k])) {
						IO.writeLn(animal[i] + " meets " + animal[k]);
						animal[i].meet(animal[k]);
						IO.writeLn("Result: "+animal[i] + ", " + animal[k]);
					}
				}
			}
			iterations++;
		}
		
		//print the results
		IO.writeLn("----------------------------------------------");
		IO.writeLn("All animals, status:");
		IO.writeLn("----------------------------------------------");
		for (int i = 0; i < lions + antilopes; i++) {
			IO.writeLn(animal[i].toString());
		}
		IO.writeLn("Number of iterations: " + iterations); // program execution is done after that.
	}

	public int livingAnimals() {
		int living = 0;
		for (int i = 0; i < animal.length; i++) {
			if (animal[i].isAnimalAlive()) {
				living++;
			}
		}
		return living;
	}
	
} // class SavannahSim